/*

  Copyright (C) 1999 President and Fellows of Harvard College

  All rights reserved.

  NOTICE: This software is provided "as is", without any warranty,
  including any implied warranty for merchantability or fitness for a
  particular purpose.  Under no circumstances shall Harvard University
  or its faculty, staff, students or agents be liable for any use of,
  misuse of, or inability to use this software, including incidental
  and consequential damages.

  License is hereby given to use, modify, and redistribute this
  software, in whole or in part, for any commercial or non-commercial
  purpose, provided that the user agrees to the terms of this
  copyright notice, including disclaimer of warranty, and provided
  that this copyright notice, including disclaimer of warranty, is
  preserved in the source code and documentation of anything derived
  from this software.  Any redistributor of this software or anything
  derived from this software assumes responsibility for ensuring that
  any parties to whom wuch a redistribution is made are fully aware of
  the terms of this license and disclaimer.

  Written by:
    Brad Karp, Harvard University EECS, April, 1999
    karp@eecs.harvard.edu

*/

#include <sys/types.h>
#include <sys/socket.h>
#include <sys/ioctl.h>
#include <net/if.h>
#include <string.h>
#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>

#include "if_iwl_ioctl.h"

main(int argc, char **argv)
{
  int s, err, sflag = 0, gflag = 0, Sflag = 0, Pflag = 0, Cflag = 0, Mflag = 0;
  int Parg, Carg, Marg, i;
  struct ifreq r;
  struct iwl_hermes_tallies tal;
  int optch;
  char *iname = "iwl0";
  char nambuf[33];

  while ((optch = getopt(argc, argv, "i:sgS:P:C:M:")) != -1)
    switch(optch) {
    case 'i':
      iname = optarg;
      break;
    case 's':
      sflag = 1;
      break;
    case 'g':
      gflag = 1;
      break;
    case 'S':
      strncpy(nambuf, optarg, sizeof(nambuf));
      Sflag = 1;
      break;
    case 'P':
      Parg = atoi(optarg);
      Pflag = 1;
      break;
    case 'C':
      Carg = atoi(optarg);
      Cflag = 1;
      break;
    case 'M':
      Marg = atoi(optarg);
      Mflag = 1;
      break;
    default:
      fprintf(stderr, "usage: %s [-i ifname] [-sg] [-S SSID] [-P port type] [-C channel] [-M 0 | 1]\n", argv[0]);
      break;
    }

  strncpy(r.ifr_name, iname, sizeof(r.ifr_name));
  s = socket(PF_INET, SOCK_DGRAM, 0);

  if (sflag) {
    r.ifr_data = (caddr_t) &tal;
    err = ioctl(s, SIOCGIWLSTATS, (caddr_t) &r);
    if (err != 0)
      perror("ioctl");
    else {
      printf("iwl0 statistics:\n");
      printf("\tUnicast packets tx: %lu\n", tal.TxUnicastFrames);
      printf("\tMulticast packets tx: %lu\n", tal.TxMulticastFrames);
      printf("\tFragments tx: %lu\n", tal.TxFragments);
      printf("\tUnicast bytes tx: %lu\n", tal.TxUnicastOctets);
      printf("\tMulticast bytes tx: %lu\n", tal.TxMulticastOctets);
      printf("\tDeferred tx: %lu\n", tal.TxDeferredTransmissions);
      printf("\tSingle retry tx: %lu\n", tal.TxSingleRetryFrames);
      printf("\tMultiple retry tx: %lu\n", tal.TxMultipleRetryFrames);
      printf("\tRetry limit exceeded tx: %lu\n", tal.TxRetryLimitExceeded);
      printf("\tDiscarded tx: %lu\n", tal.TxDiscards);
      printf("\n\tUnicast packets rx: %lu\n", tal.RxUnicastFrames);
      printf("\tMulticast packets rx: %lu\n", tal.RxMulticastFrames);
      printf("\tFragments rx: %lu\n", tal.RxFragments);
      printf("\tUnicast bytes rx: %lu\n", tal.RxUnicastOctets);
      printf("\tMulticast bytes rx: %lu\n", tal.RxMulticastOctets);
      printf("\tFCS errors rx: %lu\n", tal.RxFCSErrors);
      printf("\tNIC Buffer full rx: %lu\n", tal.RxDiscards_NoBuffer);
      printf("\tWrong SA tx: %lu\n", tal.TxDiscardsWrongSA);
      printf("\tUndecryptable rx: %lu\n", tal.RxWEPUndecryptable);
      printf("\tMsg in msg fragments rx: %lu\n", tal.RxMsgInMsgFragments);
      printf("\tMsg in bad msg fragments rx: %lu\n", tal.RxMsgInBadMsgFragments);
    }
  }

  if (gflag) {
    /* grab the SSID */
    r.ifr_data = (caddr_t) nambuf;
    err = ioctl(s, SIOCGIWLSSID, (caddr_t) &r);
    if (err != 0) {
      perror("ioctl");
      exit(-1);
    }
    else
      printf("SSID: %s\n", nambuf);

    /* grab the port type */
    r.ifr_data = (caddr_t) &i;
    err = ioctl(s, SIOCGIWLPTYPE, (caddr_t) &r);
    if (err != 0) {
      perror("ioctl");
      exit(-1);
    }
    else
      printf("port type: %d\n", i);

    /* grab the channel */
    err = ioctl(s, SIOCGIWLCHAN, (caddr_t) &r);
    if (err != 0) {
      perror("ioctl");
      exit(-1);
    }
    else
      printf("channel: %d\n", i);

    /* grab power management setting */
    err = ioctl(s, SIOCGIWLPWR, (caddr_t) &r);
    if (err != 0) {
      perror("ioctl");
      exit(-1);
    }
    else
      printf("power management: %d\n", i);

    /* grab rtx short */
    err = ioctl(s, SIOCGIWLRTXS, (caddr_t) &r);
    if (err != 0) {
      perror("ioctl");
      exit(-1);
    }
    else
      printf("max rtx for short packets: %d\n", i);

    /* grab rtx long */
    err = ioctl(s, SIOCGIWLRTXL, (caddr_t) &r);
    if (err != 0) {
      perror("ioctl");
      exit(-1);
    }
    else
      printf("max rtx for long packets: %d\n", i);
  }

  if (Sflag) {
    r.ifr_data = (caddr_t) nambuf;
    err = ioctl(s, SIOCSIWLSSID, (caddr_t) &r);
    if (err != 0) {
      perror("ioctl");
      exit(-1);
    }
    else
      printf("SSID set to %s\n", nambuf);
  }

  if (Pflag) {
    r.ifr_data = (caddr_t) &Parg;
    err = ioctl(s, SIOCSIWLPTYPE, (caddr_t) &r);
    if (err != 0) {
      perror("ioctl");
      exit(-1);
    }
    else
      printf("port type set to %d\n", Parg);
  }

  if (Cflag) {
    r.ifr_data = (caddr_t) &Carg;
    err = ioctl(s, SIOCSIWLCHAN, (caddr_t) &r);
    if (err != 0) {
      perror("ioctl");
      exit(-1);
    }
    else
      printf("channel set to %d\n", Carg);
  }

  if (Mflag) {
    r.ifr_data = (caddr_t) &Marg;
    err = ioctl(s, SIOCSIWLPWR, (caddr_t) &r);
    if (err != 0) {
      perror("ioctl");
      exit(-1);
    }
    else
      printf("power management set to to %s\n", Marg ? "ON" : "OFF");
  }

}
