/*

  Copyright (C) 1999 President and Fellows of Harvard College

  All rights reserved.

  NOTICE: This software is provided "as is", without any warranty,
  including any implied warranty for merchantability or fitness for a
  particular purpose.  Under no circumstances shall Harvard University
  or its faculty, staff, students or agents be liable for any use of,
  misuse of, or inability to use this software, including incidental
  and consequential damages.

  License is hereby given to use, modify, and redistribute this
  software, in whole or in part, for any commercial or non-commercial
  purpose, provided that the user agrees to the terms of this
  copyright notice, including disclaimer of warranty, and provided
  that this copyright notice, including disclaimer of warranty, is
  preserved in the source code and documentation of anything derived
  from this software.  Any redistributor of this software or anything
  derived from this software assumes responsibility for ensuring that
  any parties to whom wuch a redistribution is made are fully aware of
  the terms of this license and disclaimer.

  Ported from Linux and extended by:
    Brad Karp, Harvard University EECS, April, 1999
    karp@eecs.harvard.edu

  The above copyright and licensing notice applies to the
  *modifications* made at Harvard to port and extend the Linux driver,
  whose original GPL license appears below. Note that the GPL license
  is also in force on this FreeBSD port of the driver.

*/

/* This file is part of the Hardware Control Functions Light
   (HCF-light) library to control the Lucent Technologies WaveLAN/IEEE
   Network I/F Card.  The HCF is the implementation of the Wireless
   Connection I/F (WCI).
   
   The HCF-light files are a subset of the HCF files. The complete set
   offers a number of additional facilities, e.g. firmware download,
   Etherner-II encapsulation, additional diagnostic facilities, ASSERT
   logic to support debugging, 802.11 support, Configuration
   Management.  This complete set is explicitely not in the Public
   Domain but can be made available under certain restriction. (see
   the pointer below for support)
   
   The HCF-light files are free software; you can redistribute it
   and/or modify it under the terms of the GNU General Public License
   as published by the Free Software Foundation; either version 2 of
   the License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software Foundation,
   Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. 
   
   At the time of this writing, you can request for support at:
   betasupport@wavelan.com
   
   Documentation is expected to be available in the week of 8 February 1999
*/


#ifndef IWL_OSINDEP_H
#define IWL_OSINDEP_H 1

/***************************************************************************
*
* FILE	 : hcf.h *************** 2.0 
*
* DATE   : $Date:   02 Feb 1999 14:32:30  $   $Revision:   1.0  $
*
* AUTHOR : Nico Valster
*
* DESC   : Definitions and Prototypes for MSF as well as HCF sources
*
*			Customizable via HCFCFG.H
*
*
* COPYRIGHT (c) 1996, 1997, 1998 by Lucent Technologies.
*	 All Rights Reserved.
****************************************************************************/

#include <i386/isa/if_iwl_hcfcfg.h>
#include <stddef.h>
#include <i386/isa/if_iwl_mdd.h>
#include <i386/isa/if_iwl_ioctl.h>

/****************************************************************************/
/******************   H C F  F U N C T I O N   P A R A M E T E R	 ****/
/****************************************************************************/
/* offsets for hcf_put_data and hcf_get_data */

/* 802.3/E-II/802.11 offsets to access Hermes control fields */
#define HFS_STAT -0x2E	/* 0x0000 */
#define HFS_STAT_ERR RX_STAT_ERR /* link "natural" HCF name to "natural" MSF
				    name */
#define HFS_Q_INFO -0x28 /* 0x0006 */
#define HFS_TX_CNTL -0x22 /* 0x000C */
#define HFS_FRAME_CNTL -0x20 /* 0x000E */
#define HFS_ID -0x1E /* 0x0010 */

/* 802.11 relative offsets to access 802.11 header fields */
#define HFS_ADDR_1 0x00	/* 0x0012 */
#define HFS_ADDR_2 0x06	/* 0x0018 */
#define HFS_ADDR_3 0x0C	/* 0x001E */
#define HFS_SEQ_CNTL 0x12 /* 0x0024 */
#define HFS_ADDR_4 0x14	/* 0x0026 */
#define HFS_DAT_LEN 0x1A /* 0x002C */

/* 802.3 / E-II relative offsets to access 802.3 header fields */
#define HFS_ADDR_DEST 0x00 /* 0x002E */
#define HFS_ADDR_SRC 0x06 /* 0x0034 */
#define HFS_LEN	0x0C /* 0x003A */
#define HFS_DAT 0x0E /* 0x003C */

/* E-II relative offsets to access SNAP header fields */
#define HFS_TYPE 0x14 /* 0x0042	Eternet-II type in 1042/Bridge-Tunnel
			 encapsulated frame */

/* #define HCF_ACT_INT_PENDING 0x0001 /* interrupt pending, return status
                                         HCF_ACT_INT_OFF */


/****************************************************************************/
/****************   H C F  F U N C T I O N   R E T U R N   C O D E S   ******/
/****************************************************************************/

/* Debug Purposes only!! */
#define HREG_EV_TICK 0x8000 /* WMAC Controller Auxiliary Timer Tick */
#define HREG_EV_RES 0x4000 /* WMAC Controller H/W error (Wait Time-out) */
#define HREG_EV_INFO_DROP 0x2000 /* WMAC did not have sufficient RAM to build
				    Unsollicited Frame */
#define HREG_EV_NO_CARD	0x0800 /* PSEUDO event: card removed */
#define HREG_EV_DUIF_RX 0x0400 /* PSEUDO event: WMP frame received */
#define HREG_EV_INFO 0x0080 /* WMAC Controller Asynchronous Info Frame */
#define HREG_EV_CMD 0x0010 /* WMAC Controller Command completed, Status and
			      Response avaialble */
#define HREG_EV_ALLOC 0x0008 /* WMAC Controller Asynchronous part of
				Allocation/Reclaim completed */
#define HREG_EV_TX_EXC 0x0004 /* WMAC Controller Asynchronous Transmission
				 unsuccessful completed */
#define HREG_EV_TX 0x0002 /* WMAC Controller Asynchronous Transmission
			     successful completed */
#define HREG_EV_RX 0x0001 /* WMAC Controller Asynchronous Receive Frame */

typedef struct iwl_hermes_tallies CFG_HERMES_TALLIES_STRCT;

/* Note this way to define CFG_TALLIES_STRCT_SIZE implies that all tallies
   must keep the same (uint32_t) size */
#define HCF_NIC_TAL_CNT	(sizeof(CFG_HERMES_TALLIES_STRCT)/ sizeof(uint32_t))
#define	HCF_TOT_TAL_CNT	(HCF_NIC_TAL_CNT)

/***************************************************************************/
/***********   W C I    F U N C T I O N S    P R O T O T Y P E S   *********/
/***************************************************************************/

#define IFB_VERSION 0x82 /* initially 80, to be incremented by every IFB
			    layout change */

/* identifier IFB_STRCT on typedef line needed to get the individual fields
   in the MS Browser DataBase */
/* I/F Block */
typedef struct IFB_STRCT{
  /* MSF readable part of Result block structure */
  hcf_io IFB_IOBase;		/* I/O address of Hermes chip as passed by
				   MSF at hcf_connect call */
#if defined HCF_PORT_IO
  uint16_t IFB_IOBase_pad;	/* Optional field, makes IFB-layout
				   independent of IFB_IOBase size */
#endif /* HCF_PORT_IO */
  uint16_t IFB_IORange;		/* I/O Range used by Hermes chip */
  uint8_t IFB_Version;		/* initially 0, to be incremented by every IFB
				   layout change */
  uint8_t IFB_Slack_2;		/* align/slack space */
  uint8_t	IFB_HCFVersionMajor;	/* Major version of the HCF.0x01 for this
				   release */
  uint8_t IFB_HCFVersionMinor;	/* Minor version of the HCF. Incremented for
				   each coding maintenance cycle. 0x01 for the
				   Initial release */
  CFG_HERMES_TALLIES_STRCT IFB_NIC_Tallies; /* Hermes tallies */

/* part I (survives hcf_disable) */
  uint16_t IFB_CardStat;		/* see Design spec */
  uint16_t IFB_FSBase;		/* frame type dependent offset (HFS_ADDR_1_ABS
				   or HFS_ADDR_DEST_ABS) */
  uint16_t IFB_RxFence;		/* frame type dependent gap fence
				   (HFS_ADDR_DEST_ABS or HFS_LEN_ABS) */
  uint16_t IFB_IntOffCnt;		/* see Design spec */
  uint32_t IFB_TickIni;		/* initialization of counter for 1 ms
				   processor loop */

/* keep this unsigned otherwise the "clever" ASSERT in hcf_disable 
 * has a higher risk to get into trouble on slow machines
 * keep this uint16_t to prevent a "close to infinity" time out if
 * calibration fails on 32-bits machine */
  uint16_t IFB_Magic;		/* see Design spec */
  uint16_t IFB_Slack_4[2];	/* align/slack space */

/* part II (cleared or re-initialized at hcf_disable/hcf_enable) */
  uint8_t IFB_PIFRscInd;		/* see Design spec ?Q:int better than uint8_t
				   A: No! */
  uint8_t IFB_DUIFRscInd;		/* Value indicating the command resource
				   availability for the Driver-Utility I/F
				   (i.e. hcf_send_diag_msg). */

/* Values: */
/*
  No command resource		0
  Command resource available	01h-FFh
  */
  uint8_t IFB_NotifyRscInd;	/* see Design spec ?Q:int better than uint8_t
				   A: No! */
  uint8_t	IFB_Slack_6;		/* align/slack space */
  uint16_t IFB_PIF_FID;		/* see Design spec */
  uint16_t IFB_DUIF_FID;		/* field which contains FID value identifying
				   the Tx Frame Structure, to be used by
				   hcf_send_diag_msg */
  uint16_t IFB_Notify_FID;	/* field which contains FID value identifying
				   the Notify Frame Struct to be used by
				   hcf_put_info in case of Notify type codes */
  uint16_t IFB_RxFID;		/* see Design spec */
  uint16_t IFB_MB_FID;		/* pass appropriate FID to hcf_put_mb_info */
  uint16_t IFB_TxFrameType;	/* see Design spec */
  uint16_t IFB_RxLen;		/* see Design spec */
  uint16_t IFB_RxStat;		/* see Design spec */
  uint16_t IFB_UnloadIdx;		/* see Design spec */
  uint16_t IFB_PIFLoadIdx;	/* see Design spec */
  uint8_t IFB_TxCntl[2];		/* contents of HFS_TX_CNTL field of TFS:
				   0: MACPort, 1: StrucType,TxEx,TxOK */
  uint16_t IFB_BAP_0[2];		/* offset RID/FID */
  uint16_t IFB_BAP_1[2];		/* offset RID/FID */
  uint16_t IFB_IntEnMask;		/* see Design spec */
  uint16_t IFB_TimStat;		/* BAP initialization or Cmd Completion
				   failed once */
} IFB_STRCT;

typedef IFB_STRCT * IFBP;

int hcf_action(IFBP ifbp, hcf_action_cmd cmd);
void hcf_assert(IFBP ifbp, wci_bufp file_name, unsigned int line_number,
		int q);
void hcf_connect (IFBP ifbp, hcf_io io_base);
int hcf_disable (IFBP ifbp, uint16_t port);
void hcf_disconnect (IFBP ifbp);
int hcf_enable (IFBP ifbp, uint16_t port);
int hcf_get_info (IFBP ifbp, LTVP ltvp);
int hcf_get_data (IFBP ifbp, int offset, wci_bufp bufp, int len);
int hcf_service_nic (IFBP ifbp);
void hcf_put_data (IFBP ifbp, wci_bufp bufp, int len, uint16_t port);
int hcf_put_info (IFBP ifbp, LTVP ltvp);
int hcf_put_header (IFBP ifbp, int offset, wci_bufp bufp, int len,
		    uint8_t check);
int hcf_send (IFBP ifbp, uint16_t type);
int hcf_send_diag_msg (IFBP ifbp, uint16_t type, wci_bufp bufp, int len);

#endif  /* IWL_OSINDEP_H */
